#! /usr/bin/env python3

from .nca_loss import NCALoss
from .weight_regularizer_mixin import WeightRegularizerMixin
from ..utils import loss_and_miner_utils as lmu
import torch

class ProxyNCALoss(WeightRegularizerMixin, NCALoss):
    def __init__(self, num_classes, embedding_size, **kwargs):
        super().__init__(**kwargs)
        self.proxies = torch.nn.Parameter(torch.randn(num_classes, embedding_size))
        self.proxy_labels = torch.arange(num_classes)
        
    def compute_loss(self, embeddings, labels, indices_tuple):
        if self.normalize_embeddings:
            prox = torch.nn.functional.normalize(self.proxies, p=2, dim=1)
        else:
            prox = self.proxies
        nca_loss = self.nca_computation(embeddings, prox, labels, self.proxy_labels.to(labels.device), indices_tuple)
        reg_loss = self.regularization_loss(self.proxies)
        return nca_loss + reg_loss



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pytorch-metric-learning",
    version="0.9.67",
    author="Kevin Musgrave",
    author_email="tkm45@cornell.edu",
    description="The easiest way to use metric learning in your application. Modular, flexible, and extensible. Written in PyTorch.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/KevinMusgrave/pytorch-metric-learning",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.0',
    install_requires=[
		  'numpy',
		  'scikit-learn',
		  'tqdm',
          'torch',
          'torchvision',
    ],
)